<?php 
/* 
Plugin Name: Thumb-in-Post
Version: 0.7
Plugin URI: 
Description: Uses custom tags to insert thumbnails, images and objects (Flash/SVG/Quicktime) with or without captions into posts. Code and thumbnails are automatically generated. Inserts three buttons onto the quicktag menu which automate generation of the custom tags. Example custom tag for insertion of a thumbnail with caption: <code>&lt;!--wp:thumb image.jpg?cap=This is a caption wp:thumb--&gt;</code>. 
Author: Ali Hassan 
Author URI: http://www.ali-hassan.net
*/ 


// These variables set the various settings, change them to suit your Wordpress setup

function ali_getImageSettings () {

	// set url at which the images are located -- this is where the pictures should be located.
	// By default it is set to the file upload url (set it from 'miscellaneous' in Wordpress admin)
	$imgurl = get_settings('fileupload_url').'/';	

	// set url at which the thumbnails are located
	$thumburl = $imgurl;

	// set the path to the images
	$imgpath =  get_settings('fileupload_realpath').'/';

	// subdirectory of the $imgpath in which the thumbnails will be stored
	$thumbpath = $imgpath;

	// the prefix which will be added to the filename when the thumbnail is generated. Change it to whatever you like.
	$thumbprefix = "thumb-";

	// the tags for the different types of picture/object custom tag
	$thumbTag = "wp:thumb";
	$picTag = "wp:pic";
	$objectTag = "wp:object";

	// the string that will indicate where the caption begins in the tag. Don't change it to a single character ('.' would be really silly).
	$captionIndicator = " cap=";
	$srcIndicator = " src=";

	// the html code which will be used to put a div element around the thumbnail and its caption
	// the div will also being given a class based on the image filename (less the filetype extension)
	$divstart = '<div class="thumb"';

	// the html code which will be used to put a div element around the picture and its caption
	$picdivstart = '<div class="picture"';

	// the html code which will be used to put a div element around the object and its caption
	$objectdivstart = '<div class="object"';

	// and the closing div tag
	$divend = '</div>';

	// the html code that will be put around the caption
	$captagstart = '<p>';
	$captagend = '</p>';

	// set the width of the thumbnail
	$img_out_width = 400;

	// set the jpeg quality of the thumbnail -- 0 is worst; 100 is best
	$jpegQuality = 60;

	// determine what the target window shouold be: _new opens in a new window. Set to target="_self" to open in the same window
	// Set this to "" if you are using XHTML Strict.
	$target = 'target="_new"';

	// The code that will be used to indicate that the picture can be enlarged.
	//$enlarge = '<img src="'.$imgurl.'Magnify.gif" alt="Enlarge"></img>';
	$enlarge = '[Enlarge] ';

	return array ($imgurl, $thumburl, $imgpath, $thumbpath, $thumbprefix, $captionIndicator, $srcIndicator, $divstart, $divend, $captagstart, 
			$captagend, $img_out_width, $jpegQuality, $target, $enlarge, $picdivstart, $objectdivstart, $thumbTag, $picTag, $objectTag);
};

//------------------------------------settings end------------------------------------




//--------------------------------thumb-in-post starts--------------------------------

function thumbinpost($data) {

//import the settings from the getImageSettings function
list ($imgurl, $thumburl, $imgpath, $thumbpath, $thumbprefix, $captionIndicator, $srcIndicator, $divstart, $divend, $captagstart, $captagend, $img_out_width, $jpegQuality, $target, $enlarge, $picdivstart, $objectdivstart, $thumbTag, $picTag, $objectTag) = ali_getImageSettings ();

// set the tag that will be used to identify where the picture info is
$tag = $thumbTag;

//This While statement starts looping through $Data looking for the custom tag ($tag)

	while($tagstart = strpos($data, "<!--".$tag." ")) { 
		
		// this calls the function getTagFromData which retrieves the image tag from $data
		list ($entiretag, $imageFilename, $caption, $divIdName, $alt) = alitest_getTagFromData($data, $tag);

//This is where the image resize stuff starts

//If a thumbnail already exists then don't bother generating a new one	
if (!file_exists($thumbpath.$thumbprefix.$imageFilename)) {

	// retrieve file info -- check to see what file type it is
	$imginfo = getimagesize($imgpath.$imageFilename);

	// load the image from file into a variable called $img_in
	switch ($imginfo[2]) { 

		case 2:		// jpg
			$img_in = imagecreatefromjpeg($imgpath.$imageFilename);
      			break;

		case 3:		// png
			$img_in = imagecreatefrompng($imgpath.$imageFilename);
			break;

		default:
			echo('Error: cannot resize images other than JPEG or PNG; or file not found');
	};

	// figure out what the height of the output image should be in order to maintain the proper aspect ratio
	$img_ratio = $imginfo[1]/$imginfo[0];
	$img_out_height = $img_ratio*$img_out_width;

	// create a new image in $img_out with the dimensions $img_out_width and $img_out_height
	$img_out = imagecreatetruecolor($img_out_width, $img_out_height);

	// now copy $img_in into $img_out
	imagecopyresampled($img_out,$img_in,0,0,0,0,imagesx($img_out),imagesy($img_out),imagesx($img_in),imagesy($img_in));

	// write the image into a file in the thumbnail directory
	switch ($imginfo[2]) {

		case 2:		// jpg
			imagejpeg($img_out, $thumbpath.$thumbprefix.$imageFilename, $jpegQuality);
			break;

		case 3:		// png
			imagepng($img_out, $thumbpath.$thumbprefix.$imageFilename);
			break;

		default: 
			echo('Error: could not write image');
	};
	
	} // end the else statement

//Image resize ends


//Now generate the html code to insert the image into the page

	// attach an id (from $divIdName) to the divstart to give it a unique identity
	$divstartplusid = "$divstart id=\"$divIdName\">";

	// code to build links to the full size image
	$linkstart = "<a href=\"$imgurl$imageFilename\" $target>";
	$linkend = "</a>";

	// this is the code to generate the img tag
	$thumbinfo = getimagesize($thumburl.$thumbprefix.$imageFilename);
	$imgCode = "$linkstart<img src=\"$thumburl$thumbprefix$imageFilename\" alt=\"$alt\" width=\"$thumbinfo[0]\" height=\"$thumbinfo[1]\"></img>$linkend";

	// and this is the code for the caption
	$captioncode = $captagstart.$linkstart.$enlarge.$linkend.$caption.$datalength.$captagend;

	// start building the code which will replace $entiretag in $data
	if ($caption != "NONE") {
			$replacementCode = $divstartplusid.$imgCode.$captioncode.$divend;
			} else 
			{
			$replacementCode = $divstartplusid.$imgCode.$divend;
			};

	$data = str_replace ($entiretag, $replacementCode, $data);

		}

	return $data;

	};

add_filter ('the_content','thumbinpost');

//-----------------Thumb-in-Post ends here--------------------------------------


//---------------Pic-In-Post starts here----------------------------------------

function picinpost($data) {

//import the settings from the getImageSettings function
list ($imgurl, $thumburl, $imgpath, $thumbpath, $thumbprefix, $captionIndicator, $srcIndicator, $divstart, $divend, $captagstart, $captagend, $img_out_width, $jpegQuality, $target, $enlarge, $picdivstart, $objectdivstart, $thumbTag, $picTag, $objectTag) = ali_getImageSettings ();

// set the tag that will be used to identify where the picture info is
$tag = $picTag;

// extract the image name and the caption (if present from the
while($tagstart = strpos($data, "<!--".$tag." ")) { 

	// this calls the function getTagFromData which retrieves the image tag from $data
	list ($entiretag, $imageFilename, $caption, $divIdName, $alt) = alitest_getTagFromData($data, $tag);


	//Now generate the html code to insert into the page

	// attach an id (from $divIdName) to the divstart to give it a unique identity
	$divstartplusid = "$picdivstart id=\"$divIdName\">";

	// this is the code to generate the img tag
	$imginfo = getimagesize($imgurl.$imageFilename);
	$imgCode = "<img src=\"$imgurl$imageFilename\" alt=\"$alt\" width=\"$imginfo[0]\" height=\"$imginfo[1]\"></img>";

	// and this is the code for the caption
	$captioncode = $captagstart.$caption.$captagend;

	//start building the code which will replace $entiretag in $data
	if ($caption != "NONE") {
			$replacementCode = $divstartplusid.$imgCode.$captioncode.$divend;
			} else 
			{
			$replacementCode = $divstartplusid.$imgCode.$divend;
			};

	$data = str_replace ($entiretag, $replacementCode, $data);

	}

	return $data;

	};

add_filter ('the_content','picinpost');

//--------------------pic-in-post ends--------------------------------------

//---------------object-In-Post starts here----------------------------------------

function objectinpost($data) {

//import the settings from the getImageSettings function
list ($imgurl, $thumburl, $imgpath, $thumbpath, $thumbprefix, $captionIndicator, $srcIndicator, $divstart, $divend, $captagstart, $captagend, $img_out_width, $jpegQuality, $target, $enlarge, $picdivstart, $objectdivstart, $thumbTag, $picTag, $objectTag) = ali_getImageSettings ();

// set the tag that will be used to identify where the picture info is
$tag = $objectTag;

// extract the image name and the caption (if present from the
while($tagstart = strpos($data, "<!--".$tag." ")) { 

// this calls the function getTagFromData which retrieves the image tag from $data
list ($entiretag, $imageFilename, $caption, $divIdName, $alt) = alitest_getTagFromData($data, $tag);

//Now generate the html code to insert into the page

// code to build links to the full size image
$linkstart = "<a href=\"$imgurl$imageFilename\" $target>";
$linkend = "</a>";

//Need to figure out what type of file it is
$fileExtension = substr($imageFilename, strlen($imageFilename)-3, 3);

switch ($fileExtension) {

	case "swf": 	// Flash

	         //This code for flash movies should be standards compliant
	         $objectInfo = getimagesize($imgpath.$imageFilename);
	         $objectCode = '<object classid="clsid:D27CDB6E-AE6D-11cf-96B8-444553540000" 
				codebase="http://download.macromedia.com/pub/shockwave/cabs/flash/swflash.cab#version=6,0,40,0"
				width="'.$objectInfo[0].'" height="'.$objectInfo[1].'"
				class="'.$fileExtension.'">
			<param name="movie" value="'.$imgurl.$imageFilename.'" /> 
			<param name="quality" value="high" />
			<!--[if !IE]> <-->
				<object data="'.$imgurl.$imageFilename.'" type="application/x-shockwave-flash" width="'.$objectInfo[0].'" height="'.$objectInfo[1].'" class="'.$fileExtension.'">
					<param name="quality" value="high" />
					<param name="pluginurl" value="http://www.macromedia.com/go/getflashplayer" />
			<p>Error: This object cannot be displayed.</p> </object>   <!--> <![endif]-->
		</object>';
		$captioncode = $captagstart.$linkstart.$enlarge.$linkend.$caption.$datalength.$captagend;
	break;


	case "svg":	// Scalable vector graphic

		//this code for SVG insertion is standards compliant but doesn't allow proper interactivity on IE/Win
		$objectCode ='<object type="image/svg+xml" class="'.$fileExtension.'" data="'.$imgurl.$imageFilename.'"><p>broken</p></object>';

		//this is what adobe recommends for IE/Win security related reasons (surprise!), but note that <embed> is not valid anything
		//$objectCode ='<embed type="image/svg+xml" class="'.$fileExtension.'" src="'.$imgurl.$imageFilename.'">';
	
		$captioncode = $captagstart.$linkstart.$enlarge.$linkend.$caption.$datalength.$captagend;
	break;

	case "mov";	// Apple Quicktime
		$objectCode = '<object classid="clsid:02BF25D5-8C17-4B23-BC80-D3488ABDDC6B" 
				codebase="http://www.apple.com/qtactivex/qtplugin.cab"  class="'.$fileExtension.'">
 				<param name="src" value="'.$imgurl.$imageFilename.'" />
				<param name="controller" value="true" />
				<!--[if !IE]> <-->
					<object type="video/quicktime" data="'.$imgurl.$imageFilename.'" class="'.$fileExtension.'">
						<param name="controller" value="true" />
				Error: This object cannot be displayed. </object>   <!--> <![endif]-->
			</object>';
		$captioncode = $captagstart.$caption.$captagend;

	break;
	
	default:
		$objectCode = "<p>Error: Either this filetype is unsupported at present, 
				the file cannot be found or you have used an incorrect file extension.</p>";
		$captioncode = $captagstart.$caption.$captagend;

		break;
	};

	// attach an id (from $divIdName) to the divstart to give it a unique identity
	$divstartplusid = "$objectdivstart id=\"$divIdName\">";

	//start building the code which will replace $entiretag in $data
	if ($caption != "NONE") {
			$replacementCode = $divstartplusid.$objectCode.$captioncode.$divend;
			} else 
			{
			$replacementCode = $divstartplusid.$objectCode.$divend;
			};

	$data = str_replace ($entiretag, $replacementCode, $data);

	}

return $data;

};

add_filter ('the_content','objectinpost');

//----------------------------------------------object-in-post ends-----------------


//-----------------------------------fix_wp_autop starts----------------------------

//Clean up the mess that wordpress makes of <p>'s enclosing divs

function ali_fix_wpautop ($data) {

	//fix the div's at the ends
	$data = str_replace('<p><div', '<div', $data);

	//fix the div's at the ends
	$data = str_replace('</div></p>', '</div>', $data);
	$data = str_replace("</div>\n</p>", "</div>", $data);

	return $data;
	};

add_filter ('the_content','ali_fix_wpautop');

//----------------------------------------------------fix_wp_autop ends-----------------


//-----------------------------Get info from Tag starts---------------------------------

//this is the function for retrieving the $tag from $data
function alitest_getTagFromData($data, $tag) {

list ($imgurl, $thumburl, $imgpath, $thumbpath, $thumbprefix, $captionIndicator, $srcIndicator, $divstart, $divend, $captagstart, $captagend, $img_out_width, $jpegQuality, $target, $enlarge, $picdivstart, $objectdivstart, $thumbTag, $picTag, $objectTag) = ali_getImageSettings ();


	$pattern = "/<!--$tag.*-->/U";
	$regs = array ();
	if(preg_match("/<!--$tag.*-->/U",$data,$regs)) {
			$entiretag = $regs[0];
			} else {
			echo("bugger");
			};
				
	if(preg_match("/$srcIndicator\".*\"/U", $entiretag, $regs)) {
		$imageFilename = substr($regs[0], strlen($srcIndicator)+1, strlen($regs[0])-strlen($srcIndicator)-2);
		}
	
	if(preg_match("/$captionIndicator\".*\"/U", $entiretag, $regs)) {
		$caption = substr($regs[0], strlen($captionIndicator)+1, strlen($regs[0])-strlen($captionIndicator)-2);
		$text = $caption;
		$caption = wptexturize ($text);
		} else {
		$caption = "NONE";
		}

	if(preg_match("/ alt=\".*\"/U", $entiretag, $regs)) {
		$alt = substr($regs[0], strlen($captionIndicator)+1, strlen($regs[0])-strlen($captionIndicator)-2);
		} else {
			if ($caption != "NONE") {
				$alt = $caption;
				} else 
				{
				$alt = "I'm sorry, I was too lazy to write an alt tag for this picture";
				}
		}
	
	//remove the file extension from imagefilename for use as an id for the div that will surround the picture
	$divIdName = substr($imageFilename, 0, strpos($imageFilename, "."));
	
	//send the strings back to the function that requested them
	return array ($entiretag, $imageFilename, $caption, $divIdName, $alt);
}

//----------------------------Get info from Tag ends---------------------------------


//--------------------------User interface stuff down here--------------------------
//THis is the code that puts the 'Thumbnail' button on the write/edit page
//the code is based on the edit button framework by Owen Winkler at www.asymptomatic.net/wp-hacks

//gets the callback function
add_filter('admin_footer', 'ali_thumb_button');

//this is the callback function
function ali_thumb_button() {

//import the settings from the getImageSettings function
list ($imgurl, $thumburl, $imgpath, $thumbpath, $thumbprefix, $captionIndicator, $srcIndicator, $divstart, $divend, $captagstart, $captagend, $img_out_width, $jpegQuality, $target, $enlarge, $picdivstart, $objectdivstart, $thumbTag, $picTag, $objectTag) = ali_getImageSettings ();
		
	if(strpos($_SERVER['REQUEST_URI'], 'post.php')) {
?>
<script language="JavaScript" type="text/javascript"><!--
	var toolbar = document.getElementById("ed_toolbar");
	<?php
		//Insert one of the following lines for each button you want to add.
		edit_insert_button("Thumbnail", "ali_edInsertThumb", "Insert Thumbnail");
		edit_insert_button("Picture", "ali_edInsertPicture", "Insert Picture");
		edit_insert_button("Object", "ali_edInsertObject", "Insert Object");
	?>
	//Javascript handler for the button
	function ali_edInsertThumb() {
		var myValue = prompt('Enter the filename of the image', '');
			if (myValue) {
				myValue = '<!--<?php echo ($thumbTag.$srcIndicator) ?>"'+ myValue+'"';
					var myCaption = prompt('Enter a caption for the image (Not required, leave blank if you do not want one).', '');
						if (myCaption) {
							myValue = myValue + '<?php echo $captionIndicator ?>"' + myCaption +'" -->'
							} 
							else {
								myValue = myValue + ' -->'
						}
					edInsertContent(edCanvas,myValue);
			}
		}
	function ali_edInsertPicture() {
		var myValue = prompt('Enter the filename of the image', '');
			if (myValue) {
				myValue = '<!--<?php echo ($picTag.$srcIndicator) ?>"'+ myValue+'"';
					var myCaption = prompt('Enter a caption for the image (Not required, leave blank if you do not want one)', '');
						if (myCaption) {
							myValue = myValue + '<?php echo $captionIndicator ?>"' + myCaption +'" -->'
							} 
							else {
								myValue = myValue + ' -->'
						}
					edInsertContent(edCanvas,myValue);
			}
		}
	function ali_edInsertObject() {
		var myValue = prompt('Enter the filename of the object. Currently supported filetypes are Flash (.swf), Scalable Vector Graphic (.svg) and Quicktime (.mov).', '');
			if (myValue) {
				myValue = '<!--<?php echo ($objectTag.$srcIndicator) ?>"'+ myValue+'"';
					var myCaption = prompt('Enter a caption for the object (Not required, leave blank if you do not want one)', '');
						if (myCaption) {
							myValue = myValue + '<?php echo $captionIndicator ?>"' + myCaption +'" -->'
							} 
							else {
								myValue = myValue + ' -->'
						}
					edInsertContent(edCanvas,myValue);
			}
		}
//--></script>
<?php
		}
	}

	if(!function_exists('edit_insert_button'))
	{
		//edit_insert_button: Inserts a button into the editor
		function edit_insert_button($caption, $js_onclick, $title = '')
		{
?>
		if(toolbar)
		{
			var theButton = document.createElement('input');
			theButton.type = 'button';
			theButton.value = '<?php echo $caption; ?>';
			theButton.onclick = <?php echo $js_onclick; ?>;
			theButton.className = 'ed_button';
			theButton.title = "<?php echo $title; ?>";
			theButton.id = "<?php echo "ed_{$caption}"; ?>";
			toolbar.appendChild(theButton);
	}
<?php

	}
}


?>